{-----------------------------------------------------------------------------
 Package:        MD - CP Gateway Service
 Developer(s):   Brian Juergensmeyer
 Date Created:   13-Oct-2006
 Site Name:      Hines OIFO
 Description:    This class is aggregated in uCPMessage.pas, where it is used
                 to hold incoming message data from VistA.
 History:
   $Header: /CP Modernization/CliO/source/CP Gateway/uCPParams.pas 1     10/13/06 10:55a Vhaishjuergb $
   $History: uCPParams.pas $
 * 
 * *****************  Version 1  *****************
 * User: Vhaishjuergb Date: 10/13/06   Time: 10:55a
 * Created in $/CP Modernization/CliO/source/CP Gateway
 * Initial checkin
-----------------------------------------------------------------------------}
unit uCPParams;

interface

uses
  Classes, SysUtils, IniFiles, Variants, XMLIntf;

type

  TCPParam = class(TCollectionItem)
  private
    FValue: Variant;
    FName: String;
  protected
  public
  published
    property Value: Variant read FValue write FValue;
    property Name: String read FName write FName;
  end;

  TCPParams = class(TCollection)
  private
  protected
    function GetItem(Index: Integer): TCPParam;
    procedure SetItem(Index: Integer; Value: TCPParam);
  public
    function Add: TCPParam;
    procedure Assign(Source: TPersistent); override;
    function FindItemID(ID: Integer): TCPParam;
    function GetNamePath: string; override;
    function Insert(Index: Integer): TCPParam;
    procedure ParseFromXML(const ParamsNode: IXMLNode);
    property Items[Index: Integer]: TCPParam read GetItem write SetItem; default;
  published
  end;

implementation

{ TCPParams }

function TCPParams.Add: TCPParam;
begin
  result := TCPParam(inherited Add);
end;

procedure TCPParams.Assign(Source: TPersistent);
var
  I: Integer;
  Param: TCPParam;
  SourceParams : TCPParams;
begin
  if Source is TCPParams then
  begin
    SourceParams := TCPParams(Source);

    Clear;
    for I := 0 to Count - 1 do
    begin
      Param := Add;
      Param.FValue := SourceParams[I].FValue;
      Param.DisplayName := SourceParams[I].DisplayName;
    end;
  end
  else
    inherited;
end;

function TCPParams.FindItemID(ID: Integer): TCPParam;
begin
  result := TCPParam(inherited FindItemID(ID));
end;

function TCPParams.GetItem(Index: Integer): TCPParam;
begin
  result := TCPParam(inherited GetItem(Index));
end;

function TCPParams.GetNamePath: string;
begin
  result := inherited GetNamePath;
end;

function TCPParams.Insert(Index: Integer): TCPParam;
begin
  result := TCPParam(inherited Insert(Index));
end;

procedure TCPParams.ParseFromXML(const ParamsNode: IXMLNode);
var
  I: Integer;
  Param: TCPParam;
begin
  {
    Note that this function assumes a params node in proper shape.
  }
  for I := 0 to ParamsNode.Collection.Count - 1 do
  begin
    Param := Add;
    Param.Name := ParamsNode.Collection[I].NodeName;
    Param.Value := ParamsNode.Collection[I].NodeValue;
  end;
end;

procedure TCPParams.SetItem(Index: Integer; Value: TCPParam);
begin
  inherited SetItem(Index, Value);
end;

end.
