{-----------------------------------------------------------------------------
 Package:        MD - CP Gateway Service
 Developer(s):   Brian Juergensmeyer
 Date Created:   13-Oct-2006
 Site Name:      Hines OIFO
 Description:    This will allow us to map correction types in file 703.5 and 703.6 in VistA
 History:
   $Header: /CP Modernization/CliO/source/CP Gateway/uHL7CorrectionMap.pas 3     4/21/07 11:29p Vhaishjuergb $
   $History: uHL7CorrectionMap.pas $
 * 
 * *****************  Version 3  *****************
 * User: Vhaishjuergb Date: 4/21/07    Time: 11:29p
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 * Pre Spacelabs test.
 * 
 * *****************  Version 2  *****************
 * User: Vhaishjuergb Date: 12/08/06   Time: 11:15a
 * Updated in $/CP Modernization/CliO/source/CP Gateway
 * Backing up after changes.  Not ready for a full build yet, though.
 *
 * *****************  Version 1  *****************
 * User: Vhaishjuergb Date: 10/13/06   Time: 10:55a
 * Created in $/CP Modernization/CliO/source/CP Gateway
 * Initial checkin
-----------------------------------------------------------------------------}
unit uHL7CorrectionMap;

interface

uses
  Classes, SysUtils, oInterfaces;

type
  TCorrectionType = (ctPatient, ctProvider, ctUnitOfMeasure, ctObservationTerm);

  IHL7Correction = interface
  ['{E295B72A-B785-4BDD-9F3E-BE936E271507}']
    function GetCorrectionType: TCorrectionType;
    function GetUncorrectedValue: String;
    function GetCorrectedValue: String;
    property CorrectionType: TCorrectionType read GetCorrectionType;
    property UncorrectedValue: String read GetUncorrectedValue;
    property CorrectedValue: String read GetCorrectedValue;
  end;

  IHL7CorrectionList = interface
  ['{91E39362-3411-4BAE-9CFE-85CAE9AC7487}']
    function Get(Index: Integer): IHL7Correction;
    function GetCapacity: Integer;
    function GetCount: Integer;
    procedure Put(Index: Integer; const Item: IHL7Correction);
    procedure SetCapacity(NewCapacity: Integer);
    procedure SetCount(NewCount: Integer);

    procedure Clear;
    procedure Delete(Index: Integer);
    procedure Exchange(Index1, Index2: Integer);
    function First: IHL7Correction;
    function IndexOf(const Item: IHL7Correction): Integer;
    procedure Insert(Index: Integer; const Item: IHL7Correction);
    function Last: IHL7Correction;
    function Remove(const Item: IHL7Correction): Integer;
    procedure Lock;
    procedure Unlock;
    function Find(aCorrectionType: TCorrectionType; AValue: String): IHL7Correction;
    function Correct(aCorrectionType: TCorrectionType; AUncorrectedValue: String): String;
    property Capacity: Integer read GetCapacity write SetCapacity;
    property Count: Integer read GetCount write SetCount;
    property Items[Index: Integer]: IHL7Correction read Get write Put; default;
  end;


implementation

uses
  sCPGateway, MFunStr;

type
  THL7Correction = class(TInterfacedObject, IHL7Correction)
  private
    FCorrectionType: TCorrectionType;
    FUncorrectedValue: String;
    FCorrectedValue: String;
  protected
  public
    function GetCorrectionType: TCorrectionType;
    function GetUncorrectedValue: String;
    function GetCorrectedValue: String;
  published
  end;

  THL7CorrectionList = class(TInterfaceList, IHL7CorrectionList)
  private
  protected
  public
    function Get(Index: Integer): IHL7Correction;
    function GetCapacity: Integer;
    function GetCount: Integer;
    procedure Put(Index: Integer; const Item: IHL7Correction);
    procedure SetCapacity(NewCapacity: Integer);
    procedure SetCount(NewCount: Integer);

    procedure Clear;
    procedure Delete(Index: Integer);
    procedure Exchange(Index1, Index2: Integer);
    function First: IHL7Correction;
    function IndexOf(const Item: IHL7Correction): Integer;
    procedure Insert(Index: Integer; const Item: IHL7Correction);
    function Last: IHL7Correction;
    function Remove(const Item: IHL7Correction): Integer;
    procedure Lock;
    procedure Unlock;
    function Find(aCorrectionType: TCorrectionType; AValue: String): IHL7Correction;
    function Correct(aCorrectionType: TCorrectionType; AUncorrectedValue: String): String;
  published
  end;

{ THL7Correction }

function THL7Correction.GetCorrectedValue: String;
begin
  result := FCorrectedValue;
end;

function THL7Correction.GetCorrectionType: TCorrectionType;
begin
  result := FCorrectionType;
end;

function THL7Correction.GetUncorrectedValue: String;
begin
  result := FUncorrectedValue;
end;

{ THL7CorrectionList }

procedure THL7CorrectionList.Clear;
begin
  inherited Clear;
end;

function THL7CorrectionList.Correct(aCorrectionType: TCorrectionType;
  AUncorrectedValue: String): String;
var
  Correction: IHL7Correction;
begin
  Correction := Find(aCorrectionType, AUncorrectedValue);
  if assigned(Correction) then
    result := Correction.CorrectedValue
  else
    result := AUncorrectedValue;
end;

procedure THL7CorrectionList.Delete(Index: Integer);
begin
  inherited Delete(Index);
end;

procedure THL7CorrectionList.Exchange(Index1, Index2: Integer);
begin
  inherited Exchange(Index1, Index2);
end;

function THL7CorrectionList.Find(aCorrectionType: TCorrectionType;
  AValue: String): IHL7Correction;
var
  I: Integer;
  Correction: IHL7Correction;
begin
  result := nil;
  for I := 0 to Count - 1 do
  begin
    if Items[I].QueryInterface(IHL7Correction, Correction) = 0 then
    begin
      if (Correction.CorrectionType = ACorrectionType) and
         (Correction.UncorrectedValue = AValue) then
      begin
        result := Correction;
        exit;
      end;
    end;
  end;
end;

function THL7CorrectionList.First: IHL7Correction;
begin
  result := IHL7Correction(Inherited First);
end;

function THL7CorrectionList.Get(Index: Integer): IHL7Correction;
begin
  result := IHL7Correction(inherited Get(Index));
end;

function THL7CorrectionList.GetCapacity: Integer;
begin
  result := inherited GetCapacity;
end;

function THL7CorrectionList.GetCount: Integer;
begin
  result := inherited GetCount;
end;

function THL7CorrectionList.IndexOf(const Item: IHL7Correction): Integer;
begin
  result := inherited IndexOf(Item);
end;

procedure THL7CorrectionList.Insert(Index: Integer;
  const Item: IHL7Correction);
begin
  inherited Insert(Index, Item);
end;

function THL7CorrectionList.Last: IHL7Correction;
begin
  result := IHL7Correction(Inherited Last);
end;

procedure THL7CorrectionList.Lock;
begin
  inherited Lock;
end;

procedure THL7CorrectionList.Put(Index: Integer;
  const Item: IHL7Correction);
begin
  inherited Put(Index, Item);
end;

function THL7CorrectionList.Remove(const Item: IHL7Correction): Integer;
begin
  result := inherited Remove(Item);
end;

procedure THL7CorrectionList.SetCapacity(NewCapacity: Integer);
begin
  inherited SetCapacity(NewCapacity);
end;

procedure THL7CorrectionList.SetCount(NewCount: Integer);
begin
  inherited SetCount(NewCount);
end;

procedure THL7CorrectionList.Unlock;
begin
  inherited Unlock;
end;

end.
