unit oMyBroker;

interface

uses
  SysUtils,
  Windows,
  Messages,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  RPCConf1,
  TRPCB;

type
  TMyBroker = class(TRPCBroker)
  private
    fRetList: TStringList;
  public
    constructor Create(aOwner: TComponent);
    destructor Destroy; override;
    function ConnectToServer: boolean;
    function CallServer(aParams: array of string; aMultList: TStringList): Boolean;
    property Results: TStringList read fRetList;
  end;

function NewBroker(aOwner: TWinControl): TMyBroker;

implementation

function NewBroker(aOwner: TWinControl): TMyBroker;
begin
  Result := TMyBroker.Create(aOwner);
end;

{
****************************************************** TMyBroker *******************************************************
}

constructor TMyBroker.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  fRetList := TStringList.Create;
end;

destructor TMyBroker.Destroy;
begin
  FreeAndNil(fRetList);
  inherited;
end;

function TMyBroker.CallServer(aParams: array of string; aMultList: TStringList): Boolean;
var
  i: Integer;
  j: Integer;
  aStrParam: string;
begin
  if not Connected then {let's not bang on the thing endlessly}
    begin
      fRetList.Clear;
      fRetList.Add('-1^RPC Broker is not connected');
      Result := False;
      Exit;
    end;

  RemoteProcedure := 'MDZ CLIO';

  {Load any params}
  i := 0;
  if SizeOf(aParams) > 0 then
    while i <= High(aParams) do
      begin
        aStrParam := aParams[i];
        if (Copy(aStrParam, 1, 1) = '@') and (Length(aStrParam) > 1) then // Is it passed as a reference???
          begin
            Param[i].PType := reference;
            Param[i].Value := Copy(aStrParam, 2, Length(aStrParam));
          end
        else
          begin
            Param[i].PType := literal;
            Param[i].Value := aStrParam;
          end;
        inc(i);
      end;

  {Load up the multiple if it is assigned}
  if Assigned(aMultList) then
    begin
      Param[i].PType := list;
      for j := 0 to aMultList.Count - 1 do
        Param[i].Mult[IntToStr(j)] := aMultList[j];
    end;

  try
    fRetList.Clear;
    lstCall(fRetList);
    Result := (Copy(fRetList.Text, 1, 2) <> '-1'); {Check for error flag}
  except
    on e: EBrokerError do
      begin
        fRetList.Insert(0, '-1^' + e.Message);
        Connected := False; {Force this thing to stop being STOOOPID!!!}
        Result := False;
      end;
  else
    raise;
  end;
end;

function TMyBroker.ConnectToServer: boolean;
var
  srvr: string;
  port: string;
begin
  //fRPCBroker := TRPCBroker.Create(self);

  if GetServerInfo(srvr, port) <> mrCancel then
    begin
      Server := srvr;
      ListenerPort := StrToInt(port);
    end;

  Connected := True;

  Result := Connected;
end;

end.

