unit fEditChildTerm;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ComCtrls,
  ExtCtrls,
  XMLIntf,
  iCPTerminology;

type
  TfrmEditTermChild = class(TForm)
    pnlCommand: TPanel;
    pnlValues: TPanel;
    lblEvaluation: TLabel;
    Shape6: TShape;
    rbtnExtract: TRadioButton;
    rbtnDelimited: TRadioButton;
    memStart: TMemo;
    memStop: TMemo;
    udStart: TUpDown;
    udStop: TUpDown;
    lblStart: TLabel;
    lblStop: TLabel;
    cbxDelimiter: TComboBox;
    lblDelimiter: TLabel;
    pnlUnit: TPanel;
    lblUnit: TLabel;
    Shape1: TShape;
    cbxUnit: TComboBox;
    edtDescription: TEdit;
    Label5: TLabel;
    btnCancel: TButton;
    btnOK: TButton;
    Shape2: TShape;
    procedure FormCreate(Sender: TObject);
    procedure rbtnExtractClick(Sender: TObject);
    procedure rbtnDelimitedClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
  private
    fTermChild: ITermChildPair;
    fUnit: ITerm;
    fChild: ITerm;
  public
    { Public declarations }
  end;

function EditTermChild(aTermChildPair: ITermChildPair): boolean;

implementation

uses
  uGlobalVars;

{$R *.dfm}

function EditTermChild(aTermChildPair: ITermChildPair): boolean;
var
  i: integer;
  aComp: ITermComponent;
begin
  with TfrmEditTermChild.Create(Application) do
    begin
      fTermChild := aTermChildPair;
      fChild := fTermChild.getComponentTerm;
      fUnit := fTermChild.getValueUnit;

      lblEvaluation.Font.Style := [fsBold];
      lblUnit.Font.Style := [fsBold];
      lblUnit.Caption := Format('%s Unit:', [fChild.getTerm]);

      if fChild.getDataType = dtPickList then
        begin
          lblUnit.Enabled := False;
          cbxUnit.Enabled := False;
        end
      else
        begin
          fChild.LoadComponents(cbxUnit.Items, ctUnit);
          cbxUnit.ItemIndex := -1;
          if Assigned(fUnit) then
            for i := 0 to cbxUnit.Items.Count - 1 do
              if cbxUnit.Items.Objects[i].GetInterface(ITermComponent, aComp) then
                if aComp.getComponentTerm.getID = fUnit.getID then
                  cbxUnit.ItemIndex := i;
        end;

      rbtnExtract.Checked := (fTermChild.getValueType = vtExtract);
      rbtnDelimited.Checked := (fTermChild.getValueType = vtDelimited);
      if rbtnDelimited.Checked then
        if fTermChild.getValueDelimiter = ' ' then
          cbxDelimiter.ItemIndex := 0
        else
          cbxDelimiter.ItemIndex := cbxDelimiter.Items.IndexOf(fTermChild.getValueDelimiter);

      cbxDelimiter.Enabled := rbtnDelimited.Checked;
      udStart.Position := fTermChild.getValueStartPosition;
      udStop.Position := fTermChild.getValueStopPosition;
      edtDescription.Text := fTermChild.getComponentDescription;

      Result := (ShowModal = mrOK);
      if Result then
        fTermChild.getParentTerm.setChangeMade(True);
    end;
end;

procedure TfrmEditTermChild.FormCreate(Sender: TObject);
var
  i: integer;
begin
  for i := 32 to 126 do
    cbxDelimiter.Items.Add(Chr(i));
  cbxDelimiter.Items[0] := '{Space}';
end;

procedure TfrmEditTermChild.rbtnExtractClick(Sender: TObject);
begin
  lblStart.Caption := 'Start Position:';
  lblStop.Caption := 'Stop Position:';
  lblDelimiter.Enabled := False;
  cbxDelimiter.Enabled := False;
end;

procedure TfrmEditTermChild.rbtnDelimitedClick(Sender: TObject);
begin
  lblStart.Caption := 'First Piece:';
  lblStop.Caption := 'Last Piece:';
  lblDelimiter.Enabled := True;
  cbxDelimiter.Enabled := True;
end;

procedure TfrmEditTermChild.btnOKClick(Sender: TObject);
var
  aTerm: ITermUnitPair;
begin
  try
    if (cbxUnit.Items.Count > 0) and (cbxUnit.ItemIndex < 0) then
      raise Exception.Create('Unit must be specified for this term.');

    if fTermChild.getValueType <> vtCalculation then
      if (not rbtnExtract.Checked) and (not rbtnDelimited.Checked) then
        raise Exception.Create('Must be either extract for delimited value.');

    if rbtnDelimited.Checked and (cbxDelimiter.ItemIndex < 0) then
      raise Exception.Create('Must select a delimiter.');

    if udStart.Position > udStop.Position then
      raise Exception.Create('Start position must be equal to or less than stop position');

    if edtDescription.Text = '' then
      raise Exception.Create('Description is a required field.');

    with fTermChild do
      begin
        if cbxUnit.ItemIndex > -1 then
          if cbxUnit.Items.Objects[cbxUnit.ItemIndex].GetInterface(ITermUnitPair, aTerm) then
            setValueUnit(aTerm.getComponentTerm);

        if fTermChild.getValueType <> vtCalculation then
          if rbtnExtract.Checked then
            begin
              setValueType(vtExtract);
              setValueDelimiter(' ');
              setValueStartPosition(udStart.Position);
              setValueStopPosition(udStop.Position);
            end
          else
            begin
              setValueType(vtDelimited);
              if cbxDelimiter.ItemIndex < 1 then
                setValueDelimiter(' ')
              else
                setValueDelimiter(Chr(cbxDelimiter.ItemIndex + 32));
              setValueStartPosition(udStart.Position);
              setValueStopPosition(udStop.Position);
            end;

        setComponentDescription(edtDescription.Text);
        ModalResult := mrOK;
      end;
  except
    on E: Exception do
      MessageDlg('Error encountered saving Child Term.' + #13 +
        'Error: ' + E.Message, mtError, [mbOk], 0)
  end;
end;

end.

