unit fEditConversion;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  XMLIntf,
  XMLDoc,
  StdCtrls,
  ExtCtrls,
  ComCtrls,
  iCPTerminology;

type
  TfrmEditConversion = class(TForm)
    edtPreConversionOffset: TEdit;
    lblPreConversionOffset: TLabel;
    btnCancel: TButton;
    btnOK: TButton;
    edtDescription: TEdit;
    Label4: TLabel;
    Label1: TLabel;
    edtConversionFactor: TEdit;
    Label3: TLabel;
    edtPostConversionOffset: TEdit;
    Label5: TLabel;
    edtDecimalPrecision: TEdit;
    Shape1: TShape;
    lblFormula: TLabel;
    lblInstructions: TLabel;
    udDecimalPrecision: TUpDown;
    chkbxUnitsAreEqual: TCheckBox;
    procedure btnOKClick(Sender: TObject);
    procedure UpdateFormula(Sender: TObject);
    procedure chkbxUnitsAreEqualClick(Sender: TObject);
  private
    fTermConversion: ITermUnitConversion;
    fTerm: ITerm;
    fUnit: ITerm;
    { Private declarations }
  public
    { Public declarations }
  end;

function EditConversion(aTermUnitConversion: ITermUnitConversion): boolean;

implementation

uses
  uGlobalVars;

{$R *.dfm}

function EditConversion(aTermUnitConversion: ITermUnitConversion): boolean;
begin
  with TfrmEditConversion.Create(Application) do
    try
      lblFormula.Font.Style := [fsBold];
      fTermConversion := aTermUnitConversion;
      fTerm := fTermConversion.getParentTerm;
      fUnit := fTermConversion.getComponentTerm;
      chkbxUnitsAreEqual.Checked := fTermConversion.getUnitsAreEqual;
      edtPreConversionOffset.Text := FloatToStr(fTermConversion.getConversionOffsetPre);
      edtConversionFactor.Text := FloatToStr(fTermConversion.getConversionFactor);
      edtPostConversionOffset.Text := FloatToStr(fTermConversion.getConversionOffsetPost);
      udDecimalPrecision.Position := fTermConversion.getDecimalPrecision;
      edtDescription.Text := fTermConversion.getComponentDescription;
      Result := (ShowModal = mrOK);
      if Result then
        fTerm.setChangeMade(Result);
    finally
      Free;
    end;
end;

procedure TfrmEditConversion.btnOKClick(Sender: TObject);
begin
  try
    fTermConversion.setUnitsAreEqual(chkbxUnitsAreEqual.Checked);
    fTermConversion.setConversionOffsetPre(StrToFloat(edtPreConversionOffset.Text));
    fTermConversion.setConversionFactor(StrToFloat(edtConversionFactor.Text));
    fTermConversion.setConversionOffsetPost(StrToFloat(edtPostConversionOffset.Text));
    fTermConversion.setDecimalPrecision(udDecimalPrecision.Position);
    fTermConversion.setComponentDescription(edtDescription.Text);
    ModalResult := mrOK;
  except
    on E: Exception do
      MessageDlg('Error saving conversion.' + #13 + e.Message, mtError, [mbOk], 0);
  end;
end;

procedure TfrmEditConversion.chkbxUnitsAreEqualClick(Sender: TObject);
begin
  if chkbxUnitsAreEqual.Checked then
    begin
      edtPreConversionOffset.Text := '0';
      edtConversionFactor.Text := '1';
      edtPostConversionOffset.Text := '0';
      udDecimalPrecision.Position := 0;
      edtDescription.Text := 'Units are equal, no conversion needed.';
    end;

  edtPreConversionOffset.Enabled := (not chkbxUnitsAreEqual.Checked);
  edtConversionFactor.Enabled := (not chkbxUnitsAreEqual.Checked);
  edtPostConversionOffset.Enabled := (not chkbxUnitsAreEqual.Checked);
  udDecimalPrecision.Enabled := (not chkbxUnitsAreEqual.Checked);
  edtDecimalPrecision.Enabled := (not chkbxUnitsAreEqual.Checked);
  edtDescription.Enabled := (not chkbxUnitsAreEqual.Checked);
end;

procedure TfrmEditConversion.UpdateFormula(Sender: TObject);
begin
  try
    {Verify that we have all the required numerics}
    StrToFloat(edtPreConversionOffset.Text);
    StrToFloat(edtConversionFactor.Text);
    StrToFloat(edtPostConversionOffset.Text);

    if chkbxUnitsAreEqual.Checked then
      lblInstructions.Caption := Format(
        '%s to %s Formula:' + #13 + #13 + '    %s = %s',
        [fTerm.getTerm, fUnit.getTerm, fUnit.getAbbreviation, fTerm.getAbbreviation])
    else
      lblInstructions.Caption := Format(
        '%s to %s Formula:' + #13 + #13 + '    %s = (%s+%s) * %s + %s',
        [fTerm.getTerm, fUnit.getTerm,
        fUnit.getAbbreviation,
          fTerm.getAbbreviation,
          edtPreConversionOffset.Text,
          edtConversionFactor.Text,
          edtPostConversionOffset.Text]);
    btnOK.Enabled := True;
  except
    lblInstructions.Caption := 'Incomplete Formula or non-numeric values entered.';
    btnOK.Enabled := False;
  end;
end;

end.

