unit fEditToDoListItem;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ExtCtrls,
  ComCtrls,
  ImgList,
  Buttons,
  ActnList,
  iCPTerminology;

type
  TfrmEditToDoListItem = class(TForm)
    memDescription: TMemo;
    rgStatus: TRadioGroup;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    lblWho: TLabel;
    lblWhere: TLabel;
    lblWhen: TLabel;
    lblDescription: TLabel;
    btnOK: TButton;
    btnCancel: TButton;
    lvToDoList: TListView;
    lblToDoList: TLabel;
    Shape2: TShape;
    lblDetailDisplay: TLabel;
    Shape1: TShape;
    actList: TActionList;
    acNew: TAction;
    acDelete: TAction;
    bbtnNew: TBitBtn;
    ImageList1: TImageList;
    bbtnDelete: TBitBtn;
    acUpdateItem: TAction;
    bbtnUpdate: TBitBtn;
    Shape3: TShape;
    procedure FormCreate(Sender: TObject);
    procedure acNewExecute(Sender: TObject);
    procedure actListUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure lvToDoListSelectItem(Sender: TObject; Item: TListItem;      Selected: Boolean);
    procedure acUpdateItemExecute(Sender: TObject);
  private
    fTerm: ITerm;
  public
    { Public declarations }
  end;

procedure EditToDoListItem(aTerm: ITerm);

implementation

uses
  uGlobalVars,
  uXML;

{$R *.dfm}

procedure EditToDoListItem(aTerm: ITerm);
begin
  with TfrmEditToDoListItem.Create(Application) do
    try
      fTerm := aTerm;
      with aTerm.getComponentEnumerator(ctToDoItem) do
        while MoveNext do
          GetCurrentComponent.AddToListView(lvToDoList);
      ShowModal;
    finally
      Free;
    end;
end;

procedure TfrmEditToDoListItem.acNewExecute(Sender: TObject);
begin
  with fTerm.NewComponent(ctToDoItem, nil) do
    begin
      AddToListView(lvToDoList);
    end;
end;

procedure TfrmEditToDoListItem.acUpdateItemExecute(Sender: TObject);
var
  aItem: ITermToDoItem;
begin
  if TObject(lvToDoList.Selected.Data).GetInterface(ITermToDoItem, aItem) then
    begin
      aItem.setStatus(TTermToDoStatus(rgStatus.ItemIndex));
      aItem.setComponentDescription(memDescription.Text);
    end;

  with lvToDoList.Items.GetEnumerator do
    try
      while MoveNext do
        if TObject(Current.Data).GetInterface(ITermToDoItem, aItem) then
          begin
            Current.SubItems[0] := TERM_TODO_STATUS[aItem.getStatus];
            Current.SubItems[1] := aItem.getComponentDescription;
          end;
    finally
      free;
    end;
end;

procedure TfrmEditToDoListItem.actListUpdate(Action: TBasicAction; var Handled: Boolean);
begin
  acDelete.Enabled := (lvToDoList.SelCount > 0);

  rgStatus.Enabled := (lvToDoList.SelCount = 1);
  memDescription.Enabled := (lvToDoList.SelCount = 1);
  acUpdateItem.Enabled := (lvToDoList.SelCount = 1);

  Handled := True;
end;

procedure TfrmEditToDoListItem.FormCreate(Sender: TObject);
begin
  lblToDoList.Font.Style := [fsBold];
  lblDetailDisplay.Font.Style := [fsBold];
  lblWho.Font.Style := [fsBold];
  lblWhere.Font.Style := [fsBold];
  lblWhen.Font.Style := [fsBold];
  lblWho.Caption := '';
  lblWhere.Caption := '';
  lblWhen.Caption := '';
  memDescription.text := '';
  MyTerminology.LoadToDoStatuses(rgStatus.Items);
end;

procedure TfrmEditToDoListItem.lvToDoListSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
var
  aItem: ITermToDoItem;
begin
  if lvToDoList.SelCount = 1 then
    begin
      if TObject(lvToDoList.Selected.Data).GetInterface(ITermToDoItem, aItem) then
        begin
          lblWho.Caption := aItem.getEnteredBy;
          lblWhere.Caption := aItem.getEnteredAtWorkstation;
          lblWhen.Caption := DateTimeToXMLString(aItem.getDateTimeEntered);
          rgStatus.ItemIndex := Ord(aItem.getStatus);
          memDescription.Text := aItem.getComponentDescription;
        end;
    end
  else
    begin
      lblWho.Caption := '';
      lblWhere.Caption := '';
      lblWhen.Caption := '';
      rgStatus.ItemIndex := -1;
      memDescription.Text := '';
    end;
end;

end.

