unit fEditUnit;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ComCtrls,
  XMLDoc,
  XMLIntf,
  ExtCtrls,
  iCPTerminology;

type
  TfrmEditUnit = class(TForm)
    btnCancel: TButton;
    btnOK: TButton;
    pnlValues: TPanel;
    Label6: TLabel;
    Label7: TLabel;
    lblValidation: TLabel;
    memVMin: TMemo;
    memVMax: TMemo;
    Shape1: TShape;
    Shape2: TShape;
    Label8: TLabel;
    memDecimal: TMemo;
    udDecimal: TUpDown;
    Label1: TLabel;
    Label2: TLabel;
    memRMin: TMemo;
    memRMax: TMemo;
    Label3: TLabel;
    Label4: TLabel;
    btnTest: TButton;
    procedure btnOKClick(Sender: TObject);
    procedure udDecimalClick(Sender: TObject; Button: TUDBtnType);
    procedure ValidateNumber(Sender: TObject);
    procedure btnTestClick(Sender: TObject);
  private
    fUnitPair: ITermUnitPair;
    fUnit: ITerm;
    fFmt: string;
  public
    { Public declarations }
  end;

function EditUnit(aUnitPair: ITermUnitPair): boolean;

implementation

uses
  uGlobalVars,
  Math;

{$R *.dfm}

function EditUnit(aUnitPair: ITermUnitPair): boolean;
begin
  with TfrmEditUnit.Create(Application) do
    begin
      fUnit := aUnitPair.getComponentTerm;
      fUnitPair := aUnitPair;
      lblValidation.Font.Style := [fsBold];
      lblValidation.Caption := 'Validation for Unit: ' + fUnit.getTerm;
      udDecimal.Position := fUnitPair.getDecPrecision;
      fFmt := Format('%%0.%df', [udDecimal.Position]);
      memVMin.Text := Format(fFmt, [fUnitPair.getMinValue]);
      memVMax.Text := Format(fFmt, [fUnitPair.getMaxValue]);
      memRMin.Text := Format(fFmt, [fUnitPair.getReferenceLow]);
      memRMax.Text := Format(fFmt, [fUnitPair.getReferenceHigh]);
      Result := (ShowModal = mrOK);
    end;
end;

procedure TfrmEditUnit.ValidateNumber(Sender: TObject);
var
  aStrVal: string;
  aDecPos: integer;
begin
  if TMemo(Sender).Text <> '' then
    try
      aStrVal := TMemo(Sender).Text;
      StrToFloat(aStrVal); {Make sure it's numeric}
      if Pos('.', aStrVal) > 0 then {Does it have a decimal?}
        begin
          aDecPos := Length(aStrVal) - Pos('.', aStrVal);
          { debug test incase it starts going stoopid again
          ShowMessageFmt(
            'Length of String: %d, Decimal Pos(aStVal): %d, Decimal Precision: %d',
            [Length(aStVal), Pos('.', aStVal), aDecPos]);
          }
          if aDecPos > udDecimal.Position then
            raise ETerminologyDictionaryError.CreateFmt('Invalid decimal precision, only %d places allowed.', [udDecimal.Position]);
        end;
      TMemo(Sender).Text := Format(Format('%%0.%df', [udDecimal.Position]), [StrToFloat(TMemo(Sender).Text)]);
    except
      on E: EConvertError do
        begin
          MessageDlg(e.Message + #13 + 'Not a valid numeric value', mtError, [mbOk], 0);
          TMemo(Sender).SelectAll;
          TMemo(Sender).SetFocus;
        end;
      on e: ETerminologyDictionaryError do
        begin
          MessageDlg(e.Message, mtError, [mbOk], 0);
          TMemo(Sender).SelectAll;
          TMemo(Sender).SetFocus;
        end;
    end;
end;

procedure TfrmEditUnit.btnOKClick(Sender: TObject);
begin
  try
    if (StrToFloat(memVMin.Text) < StrToFloat(memRMin.Text)) and
      (StrToFloat(memRMin.Text) < StrToFloat(memRMax.Text)) and
      (StrToFloat(memRMax.Text) < StrToFloat(memVMax.Text)) then
      begin
        fUnitPair.setDecPrecision(udDecimal.Position);
        fUnitPair.setMinValue(StrToFloat(memVMin.Text));
        fUnitPair.setMaxValue(StrToFloat(memVMax.Text));
        fUnitPair.setReferenceLow(StrToFloat(memRMin.Text));
        fUnitPair.setReferenceHigh(StrToFloat(memRMax.Text));
        ModalResult := mrOK;
      end
    else
      raise Exception.Create('Evaluation values contain an overlap');
  except
    on E: EConvertError do
      MessageDlg('Non-Numeric value entered.', mtError, [mbOk], 0);
    on E: Exception do
      MessageDlg(e.Message, mtError, [mbOk], 0);
  end;
end;

procedure TfrmEditUnit.btnTestClick(Sender: TObject);
var
  aStrValue: string;
  //aDblValue: Double;
  //i: integer;
begin
  if InputQuery('Test', 'Value:', aStrValue) then
    try
      StrToFloat(aStrValue);
      //Val(aStrValue, aDblValue, i);
      //if i > 0 then
      //  raise Exception.CreateFmt('Value %s is not numeric.', [aStrValue]);
    except
      on E: Exception do
        MessageDlg(E.Message, mtError, [mbOk], 0);
    end;
end;

procedure TfrmEditUnit.udDecimalClick(Sender: TObject; Button: TUDBtnType);
begin
  fFmt := Format('%%0.%df', [udDecimal.Position]);
  memVMin.Text := Format(fFmt, [StrToFloatDef(memVMin.Text, 0)]);
  memVMax.Text := Format(fFmt, [StrToFloatDef(memVMax.Text, 0)]);
  memRMin.Text := Format(fFmt, [StrToFloatDef(memRMin.Text, 0)]);
  memRMax.Text := Format(fFmt, [StrToFloatDef(memRMax.Text, 0)]);
end;

end.

