unit fExtractTerms;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ActnList,
  StdCtrls,
  CheckLst,
  iCPTerminology;

type
  TfrmExtract = class(TForm)
    clbxColumns: TCheckListBox;
    Button1: TButton;
    Button2: TButton;
    acList: TActionList;
    acExtract: TAction;
    clbxTypes: TCheckListBox;
    Label1: TLabel;
    Label2: TLabel;
    procedure acListUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure acExtractExecute(Sender: TObject);
  private
    fTerminology: ITerminologyController;
  public
    { Public declarations }
  end;

procedure ExtractTerms(aTerminology: ITerminologyController);

implementation

{$R *.dfm}

procedure ExtractTerms(aTerminology: ITerminologyController);
begin
  with TfrmExtract.Create(Application.MainForm) do
    begin
      fTerminology := aTerminology;
      fTerminology.LoadTermTypes(clbxTypes.Items);
      fTerminology.LoadTermAttributes(clbxColumns.Items);
      ShowModal;
    end;
end;

procedure TfrmExtract.acExtractExecute(Sender: TObject);
var
  i: integer;
  aTypes: set of TTermType;
  aAttribute: TTermAttribute;
  aAttributes: set of TTermAttribute;
  aLine: string;
  aList: TStringList;
begin
  aTypes := [];
  aAttributes := [];

  for i := 0 to clbxTypes.Count - 1 do
    if clbxTypes.Checked[i] then
      aTypes := aTypes + [TTermType(i)];

  for i := 0 to clbxColumns.Count - 1 do
    if clbxColumns.Checked[i] then
      aAttributes := aAttributes + [TTermAttribute(i)];

  aList := TStringList.Create;
  aLine := '';

  for aAttribute := Low(TTermAttribute) to High(TTermAttribute) do
    if aAttribute in aAttributes then
      aLine := aLine + AnsiQuotedStr(TERM_ATTRIBUTE[aAttribute], '"') + #9;

  aList.Add(aLine);

  with fTerminology.getEnumerator do
    while MoveNext do
      try
        if getCurrentTerm.getTermType in aTypes then
          begin
            aLine := '';
            for aAttribute := Low(TTermAttribute) to High(TTermAttribute) do
              if aAttribute in aAttributes then
                begin
                  if aLine <> '' then
                    aLine := aLine + #9;
                  case aAttribute of
                    taID: aLine := aLine + AnsiQuotedStr(getCurrentTerm.getID, '"');
                    taName: aLine := aLine + AnsiQuotedStr(getCurrentTerm.getTerm, '"');
                    taAbbreviation: aLine := aLine + AnsiQuotedStr(getCurrentTerm.getAbbreviation, '"');
                    taDisplayName: aLine := aLine + AnsiQuotedStr(getCurrentTerm.getDisplayName, '"');
                    taTermType: aLine := aLine + AnsiQuotedStr(TERM_TYPE[getCurrentTerm.getTermType], '"');
                    taDataType: aLine := aLine + AnsiQuotedStr(TERM_DATATYPE[getCurrentTerm.getDataType], '"');
                    taActive: aLine := aLine + AnsiQuotedStr(BOOLEAN_TO_STR[getCurrentTerm.getActive], '"');
                    taDescription: aLine := aLine + AnsiQuotedStr(getCurrentTerm.getDescription, '"');
                    taHelpText: aLine := aLine + AnsiQuotedStr(getCurrentTerm.getHelpText, '"');
                    taBooleanTrue: aLine := aLine + AnsiQuotedStr(getCurrentTerm.getBooleanValueTrue, '"');
                    taBooleanFalse: aLine := aLine + AnsiQuotedStr(getCurrentTerm.getBooleanValueFalse, '"');
                    taMultiSelect: aLine := aLine + AnsiQuotedStr(BOOLEAN_TO_STR[getCurrentTerm.getMultiSelectPicklist], '"');
                    taVUID: aLine := aLine + AnsiQuotedStr(getCurrentTerm.getVUID, '"');
                  end;
                end;
            aList.Add(aLine);
          end;
      except
        ShowMessage('Crunch');
      end;

  with TSaveDialog.Create(Self) do
    try
      if Execute then
        aList.SaveToFile(FileName);
    finally
      Free;
    end;

  ModalResult := mrOk;
end;

procedure TfrmExtract.acListUpdate(Action: TBasicAction; var Handled: Boolean);
var
  i: integer;
  aTypes: integer;
  aColumns: integer;
begin
  aTypes := 0;
  for i := 0 to clbxTypes.Items.Count - 1 do
    if clbxTypes.Checked[i] then
      inc(aTypes);
  aColumns := 0;
  for i := 0 to clbxColumns.Items.Count - 1 do
    if clbxColumns.Checked[i] then
      inc(aColumns);
  acExtract.Enabled := (aColumns > 0) and (aTypes > 0);
end;

end.

