unit fMain;

interface

uses
  XPMan,
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ExtCtrls,
  ComCtrls,
  ToolWin,
  ActnCtrls,
  ActnMan,
  ActnMenus,
  XPStyleActnCtrls,
  ActnList,
  Buttons,
  Menus,
  StdCtrls,
  ImgList,
  SyncObjs,
  XMLIntf,
  XMLDoc,
  iCPTerminology;

type
  TfrmMain = class(TForm)
    sBar: TStatusBar;
    Splitter1: TSplitter;
    actMgr: TActionManager;
    acExit: TAction;
    CoolBar1: TCoolBar;
    ActionMainMenuBar1: TActionMainMenuBar;
    ToolBar1: TToolBar;
    ToolButton2: TToolButton;
    dlgOpen: TOpenDialog;
    ilSmall: TImageList;
    pmn: TPopupMenu;
    acViewXML: TAction;
    acFindTerm: TAction;
    acViewSmallIcons: TAction;
    Details2: TMenuItem;
    LargeIcons1: TMenuItem;
    List1: TMenuItem;
    SmallIcons1: TMenuItem;
    acUpOneLevel: TAction;
    ToolButton4: TToolButton;
    pnlDisplay: TPanel;
    sbx: TScrollBox;
    bvlLeft: TBevel;
    bvlRight: TBevel;
    spltrBottom: TSplitter;
    ToolButton3: TToolButton;
    dlgFindTerm: TFindDialog;
    acSave: TAction;
    ToolButton5: TToolButton;
    acSaveAs: TAction;
    dlgSaveAs: TSaveDialog;
    acFileOpen: TAction;
    acValidateTerms: TAction;
    ToolButton6: TToolButton;
    acAddToDoItem: TAction;
    lblMessage: TLabel;
    acInstall: TAction;
    acNewTerm: TAction;
    acLoadXML: TAction;
    acClose: TAction;
    ToolButton1: TToolButton;
    acBuildHTML: TAction;
    acTermProperties: TAction;
    Properties1: TMenuItem;
    acSaveTerm: TAction;
    ToolButton7: TToolButton;
    Splitter2: TSplitter;
    Splitter3: TSplitter;
    pnlLegend: TPanel;
    Image1: TImage;
    Label1: TLabel;
    Image2: TImage;
    Label2: TLabel;
    Image3: TImage;
    Label3: TLabel;
    Image4: TImage;
    Label4: TLabel;
    Label5: TLabel;
    Shape1: TShape;
    tv: TTreeView;
    ToolButton8: TToolButton;
    acExtractTerms: TAction;
    procedure acFileOpenExecute(Sender: TObject);
    procedure acExitExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure tvChanging(Sender: TObject; Node: TTreeNode; var AllowChange: Boolean);
    procedure acUpOneLevelExecute(Sender: TObject);
    procedure tvGetImageIndex(Sender: TObject; Node: TTreeNode);
    procedure acViewXMLExecute(Sender: TObject);
    procedure acFindTermExecute(Sender: TObject);
    procedure dlgFindTermFind(Sender: TObject);
    procedure acSaveAsExecute(Sender: TObject);
    procedure acSaveExecute(Sender: TObject);
    procedure acValidateTermsExecute(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure acInstallExecute(Sender: TObject);
    procedure acNewTermExecute(Sender: TObject);
    procedure acLoadXMLExecute(Sender: TObject);
    procedure ShowStatus(aMsg: string = 'Ready');
    procedure acCloseExecute(Sender: TObject);
    procedure actMgrUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure acSaveTermExecute(Sender: TObject);
    procedure acTermPropertiesExecute(Sender: TObject);
    procedure acExtractTermsExecute(Sender: TObject);
  private
    fCurrentXMLNode: IXMLNode;
    fCurrentTerm: ITerm;
    fSearchNode: TTreeNode;
    fNodes: array[TTermType] of TTreeNode;
    procedure UpdateTerm(aTerm: ITerm);
    procedure SetCurrentXMLNode(aXMLNode: IXMLNode);
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

uses
  DateUtils,
  uGlobalVars,
  fEditToDoListItem,
  fInstaller,
  fNewTerm,
  fErrorLog,
  fExtractTerms,
  mEditTerm,
  mEditPairs,
  mEditFileHeader,
  mTermList;

{$R *.dfm}

var
  MainTermEditFrame: TfraEditTerm;
  FileHeaderFrame: TfraEditFileHeader;
  TermListerFrame: TfraTermList;
  CanInstall: Boolean = False;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  MyTerminology := NewTerminologyController;
  MyTerminology.setOnStatusUpdate(ShowStatus);

  FileHeaderFrame := NewEditFileHeaderFrame(sbx);
  TermListerFrame := NewTermListFrame(sbx);
  lblMessage.Font.Style := [fsBold];
  //Font := Screen.IconFont;
  CanInstall := FindCmdLineSwitch('useinstaller', ['/', '-'], true);
end;

procedure TfrmMain.acFileOpenExecute(Sender: TObject);
var
  aTermType: TTermType;
  aRoot: TTreeNode;
begin
  if MyTerminology.IsOpen then
    acClose.Execute;

  tv.Items.Clear;
  tv.Items.BeginUpdate;

  if dlgOpen.Execute then
    try
      MyTerminology.Load(dlgOpen.FileName);

      aRoot := tv.Items.AddChild(nil, 'Terminology');
      for aTermType := ttObservation to High(TTermType) do
        fNodes[aTermType] := tv.Items.AddChild(aRoot, TERM_TYPE_DISPLAY[aTermType]);

      with MyTerminology.getEnumerator do
        while MoveNext do
          begin
            ShowStatus(Format('Loading %d%%...', [GetPercentageDone]));
            GetCurrentTerm.AddToTreeView(tv, fNodes[GetCurrentTerm.getTermType], True);
          end;

      sBar.Panels[0].Text := 'Ready';
      sBar.Panels[1].Text := dlgOpen.FileName;
      sBar.Invalidate;
      Application.ProcessMessages;
      tv.AlphaSort;
      aRoot.Expand(False);
    except
      on E: Exception do
        begin
          ShowMessageFmt('Error: %s', [e.message]);
          tv.Items.Clear;
        end;
    end;
  tv.Items.EndUpdate;
end;

procedure TfrmMain.acExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.acExtractTermsExecute(Sender: TObject);
begin
  ExtractTerms(MyTerminology);
end;

procedure TfrmMain.tvChanging(Sender: TObject; Node: TTreeNode; var AllowChange: Boolean);
begin
  Enabled := False;
  try
    if tv.Selected <> nil then
      if Assigned(tv.Selected.Data) then
        if TObject(tv.Selected.Data).GetInterface(ITerm, fCurrentTerm) then
          if fCurrentTerm.getChangeMade then
            if MessageDlg(Format('Save changes to Term ''%s''?', [fCurrentTerm.getTerm]), mtConfirmation, [mbYes, mbNo], 0) = mrYes then
              acSaveTerm.Execute
            else
              fCurrentTerm.Load; { Reload with the XML data and go back to unchanged :) }

    pnlDisplay.Caption := '';
    sbx.Visible := False;
    lblMessage.Visible := False;

    fCurrentTerm := nil;

    FreeAndNil(MainTermEditFrame);
    FileHeaderFrame.Visible := False;
    TermListerFrame.Visible := False;

    SetCurrentXMLNode(nil);

    AllowChange := True;

    if not Assigned(Node) then
      Exit;

    if Node.IsFirstNode then
      begin
        FileHeaderFrame.Load;
        FileHeaderFrame.Visible := True;
        sbx.Visible := True;
      end
    else
      if Assigned(Node.Data) and (Node.Count = 0) then
        begin
          lblMessage.Caption := Format('Loading ''%s'', please wait...', [Node.Text]);
          lblMessage.Visible := True;
          if TObject(Node.Data).GetInterface(ITerm, fCurrentTerm) then
            begin
              SetCurrentXMLNode(fCurrentTerm.getXMLNode);
              MainTermEditFrame := NewEditTermFrame(sbx, fCurrentTerm);
              MainTermEditFrame.OnUpdateTerm := UpdateTerm;
              MainTermEditFrame.bbtnSave.Action := acSaveTerm;
              acUpOneLevel.Enabled := Assigned(Node.Parent);
              sbx.Visible := True;
            end;
          acUpOneLevel.Enabled := Assigned(Node.Parent)
        end
      else
        begin
          //ShowMessage('Load the listview frame!');
          TermListerFrame.Load(Node);
          TermListerFrame.Visible := True;
          acUpOneLevel.Enabled := Assigned(Node.Parent);
        end;
  finally
    sbx.Visible := True;
    Enabled := True;
  end;
end;

procedure TfrmMain.acUpOneLevelExecute(Sender: TObject);
begin
  tv.Selected := tv.Selected.Parent;
end;

procedure TfrmMain.tvGetImageIndex(Sender: TObject; Node: TTreeNode);
var
  aTerm: ITerm;
begin
  if Assigned(Node.Data) then
    begin
      if TObject(Node.Data).GetInterface(ITerm, aTerm) then
        if aTerm.getActive then
          begin
            if aTerm.getComponentCount(ctError) = 0 then
              begin
                Node.ImageIndex := 2;
                Node.SelectedIndex := 2;
              end
            else
              begin
                Node.ImageIndex := 12;
                Node.SelectedIndex := 12;
              end
          end
        else
          begin
            Node.ImageIndex := 11;
            Node.SelectedIndex := 11;
          end
    end
  else
    if Node.Expanded or Node.Selected then
      begin
        Node.ImageIndex := 0;
        Node.SelectedIndex := 0;
      end
    else
      begin
        Node.ImageIndex := 1;
        Node.SelectedIndex := 1;
      end;
end;

procedure TfrmMain.acViewXMLExecute(Sender: TObject);
begin
  {
  acViewXML.Checked := (not acViewXML.Checked);
  pnlXML.Visible := acViewXML.Checked;
  spltrBottom.Top := pnlXML.Top - 2;
  acLoadXML.Execute;
  }
end;

procedure TfrmMain.acFindTermExecute(Sender: TObject);
begin
  fSearchNode := tv.Items[0];
  dlgFindTerm.FindText := '';
  dlgFindTerm.Execute;
end;

procedure TfrmMain.acInstallExecute(Sender: TObject);
begin
  InstallTerminology;
end;

procedure TfrmMain.acNewTermExecute(Sender: TObject);
var
  aNewTerm: ITerm;
begin
  aNewTerm := CreateNewTerm;
  if Assigned(aNewTerm) then
    begin
      aNewTerm.Save;
      tv.Selected := aNewTerm.AddToTreeView(tv, fNodes[aNewTerm.getTermType]);
      tv.AlphaSort;
    end;
end;

procedure TfrmMain.acValidateTermsExecute(Sender: TObject);
var
  aLog: TStringList;
begin
  try
    aLog := TStringList.Create;
    with MyTerminology.getEnumerator do
      while MoveNext do
        begin
          ShowStatus(Format('Validating Terminology - %d%% Complete', [GetPercentageDone]));
          GetCurrentTerm.Validate;
          GetCurrentTerm.Save;
          getCurrentTerm.LoadErrors(aLog);
        end;

    ShowStatus;
    ViewErrorLog(aLog);
  finally
    FreeAndNil(aLog);
  end;
end;

procedure TfrmMain.dlgFindTermFind(Sender: TObject);
var
  aComp1, aComp2: string;
begin
  while fSearchNode <> nil do
    begin
      fSearchNode := fSearchNode.GetNext;

      if fSearchNode = nil then
        begin
          MessageDlg('End of terminology file reached.', mtInformation, [mbOk], 0);
          dlgFindTerm.CloseDialog;
          Exit;
        end;

      if Assigned(fSearchNode.Data) then
        begin
          if frMatchCase in dlgFindTerm.Options then
            begin
              aComp1 := fSearchNode.Text;
              aComp2 := dlgFindTerm.FindText;
            end
          else
            begin
              aComp1 := UpperCase(fSearchNode.Text);
              aComp2 := UpperCase(dlgFindTerm.FindText);
            end;

          if Pos(aComp2, aComp1) > 0 then
            begin
              tv.Selected := fSearchNode;
              Exit;
            end;
        end;
    end;
end;

procedure TfrmMain.acSaveAsExecute(Sender: TObject);
begin
  if dlgSaveAs.Execute then
    MyTerminology.SaveAs(dlgSaveAs.FileName);
end;

procedure TfrmMain.acSaveExecute(Sender: TObject);
begin
  MyTerminology.Save;
end;

procedure TfrmMain.acSaveTermExecute(Sender: TObject);
begin
  fCurrentTerm.Validate;
  if not fCurrentTerm.getIsValidDefinition then
    if MessageDlg('This term has errors in it''s definition. Save it anyway?', mtConfirmation, [mbYes, mbNo], 0) = mrNo then
      Exit;
  fCurrentTerm.Save;
  tv.Invalidate;
end;

procedure TfrmMain.acTermPropertiesExecute(Sender: TObject);
var
  aList: TStringList;
  i: integer;
  aTerm: ITerm;
begin
  aList := TStringList.Create;
  try
    with MyTerminology.getEnumerator do
      while MoveNext do
        if GetCurrentTerm.ContainsTerm(fCurrentTerm) then
          GetCurrentTerm.AddToList(aList);
    for i := 0 to aList.Count - 1 do
      if aList.Objects[i].GetInterface(ITerm, aTerm) then
        aList[i] := Format('%s: %s', [TERM_TYPE_DISPLAY[aTerm.getTermType], aTerm.getTerm]);
    aList.Sorted := True;
    if aList.Count > 0 then
      MessageDlg('''' + fCurrentTerm.getTerm + ''' is a component of the following terms: ' + #13 + #13 + aList.Text, mtInformation, [mbOk], 0)
    else
      MessageDlg('''' + fCurrentTerm.getTerm + ''' is not a component of any term.', mtInformation, [mbOk], 0);
  finally
    FreeAndNil(aList);
  end;
end;

procedure TfrmMain.actMgrUpdate(Action: TBasicAction; var Handled: Boolean);
begin
  acExtractTerms.Enabled := MyTerminology.IsOpen;
  acFindTerm.Enabled := MyTerminology.IsOpen;
  acViewXML.Enabled := MyTerminology.IsOpen;
  acSave.Enabled := MyTerminology.IsOpen;
  acSaveAs.Enabled := MyTerminology.IsOpen;
  acValidateTerms.Enabled := MyTerminology.IsOpen;
  acInstall.Enabled := MyTerminology.IsOpen and CanInstall;
  acNewTerm.Enabled := MyTerminology.IsOpen;
  acClose.Enabled := MyTerminology.IsOpen;
  sBar.Panels[1].Text := MyTerminology.getFileName;

  if Assigned(fCurrentTerm) then
    begin
      acSaveTerm.Enabled := fCurrentTerm.getChangeMade;
      acTermProperties.Enabled := True;
    end
  else
    begin
      acSaveTerm.Enabled := False;
      acTermProperties.Enabled := False;
    end;

  if tv.Selected <> nil then
    acUpOneLevel.Enabled := (tv.Selected.Parent <> nil)
  else
    acUpOneLevel.Enabled := False;

  Handled := True;
end;

procedure TfrmMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if Assigned(MyTerminology) then
    if MyTerminology.getModified then
      if MessageDlg('Save changes before closing?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
        MyTerminology.Save;
  CanClose := True;
end;

procedure TfrmMain.UpdateTerm(aTerm: ITerm);
begin
  if Assigned(aTerm) then
    begin
      tv.Selected.Text := aTerm.getTerm;
      tv.Invalidate;
      {
      if pnlXML.Visible then
        acLoadXML.Execute;
      }
    end;
end;

procedure TfrmMain.SetCurrentXMLNode(aXMLNode: IXMLNode);
begin
  fCurrentXMLNode := aXMLNode;
  acLoadXML.Execute;
end;

procedure TfrmMain.acLoadXMLExecute(Sender: TObject);
begin
  {
  if Assigned(fCurrentXMLNode) and pnlXML.Visible then
    memXML.Lines.Text := FormatXMLData(fCurrentXMLNode.XML)
  else
    memXML.Lines.Text := 'No XML data for this item';
  }
end;

procedure TfrmMain.ShowStatus(aMsg: string = 'Ready');
begin
  sBar.Panels[1].Text := aMsg;
  Application.ProcessMessages;
end;

procedure TfrmMain.acCloseExecute(Sender: TObject);
begin
  if MyTerminology.IsOpen then
    begin
      if Assigned(fCurrentTerm) then
        if fCurrentTerm.getChangeMade then
          if MessageDlg('Save changes to term ' + fCurrentTerm.getTerm + '?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
            acSaveTerm.Execute;

      if MyTerminology.getModified then
        if MessageDlg('Save changes to ' + MyTerminology.getFileName + ' before closing?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
          MyTerminology.Save;

      tv.Selected := nil;
      tv.Items.Clear;
      MyTerminology.Close;
    end;
end;

end.

