unit mEditTerm;
================================================================================
*       Application:  Clinical Observations
*       Revision:     $Revision: 17 $  $Modtime: 6/25/09 1:59p $
*       Developer:      
*       Site:         Hines OIFO
*       Description:  Terminology editor.
*       Notes:        THIS APPLICATION IS NOT TO BE RELEASED TO THE FIELD.
*                     ***TERMINOLOGY FILES ARE UNEDITABLE AFTER RELEASE***
================================================================================
*       $Archive: /CP Modernization/CliO/source/CP Terminology Editor/mEditTerm.pas $
================================================================================
interface
uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  XMLDoc,
  XMLIntf,
  StdCtrls,
  ExtCtrls,
  ComCtrls,
  Buttons,
  Menus,
  iCPTerminology;
type
  TOnUpdateTerm = procedure(aTerm: ITerm) of object;
  TfraEditTerm = class(TFrame)
    Label1: TLabel;
    edtDisplayName: TEdit;
    Label2: TLabel;
    edtAbbreviation: TEdit;
    cbxActive: TCheckBox;
    Label3: TLabel;
    Label4: TLabel;
    Shape1: TShape;
    pnlMain: TPanel;
    cbxDataType: TComboBox;
    lblDataType: TLabel;
    pnlBoolean: TPanel;
    Label5: TLabel;
    Label6: TLabel;
    edtBooleanValueTrue: TEdit;
    edtBooleanValueFalse: TEdit;
    lblBoolean: TLabel;
    Shape2: TShape;
    Label8: TLabel;
    edtName: TEdit;
    cbxMultiSelect: TCheckBox;
    pnlCommands: TPanel;
    lblCommands: TLabel;
    Shape3: TShape;
    btnAssignVuid: TButton;
    btnToDo: TButton;
    btnValidate: TButton;
    btnHTML: TButton;
    shpComponents: TShape;
    lblComponents: TLabel;
    memDescription: TMemo;
    memHelpText: TMemo;
    pnlComponents: TPanel;
    lblFolder: TLabel;
    bbtnSave: TBitBtn;
    procedure cbxDataTypeChange(Sender: TObject);
    procedure btnToDoClick(Sender: TObject);
    procedure btnAssignVuidClick(Sender: TObject);
    procedure btnValidateClick(Sender: TObject);
    procedure ChangeMade(Sender: TObject);
  private
    fLoading: boolean;
    fTerm: ITerm;
    fOnUpdateTerm: TOnUpdateTerm;
    fOnStatusUpdate: TOnStatusUpdate;
    procedure LoadTerm(aTerm: ITerm);
  public
    property OnUpdateTerm: TOnUpdateTerm read fOnUpdateTerm write fOnUpdateTerm;
    property OnStatusUpdate: TOnStatusUpdate read fOnStatusUpdate write fOnStatusUpdate;
  end;
function NewEditTermFrame(aOwner: TWinControl; aTerm: ITerm): TfraEditTerm;
implementation
uses
  uXML,
  fEditToDoListItem,
  uGlobalVars,
  fErrorLog,
  mEditPairs;
{$R *.dfm}
function NewEditTermFrame(aOwner: TWinControl; aTerm: ITerm): TfraEditTerm;
begin
  Result := TfraEditTerm.Create(aOwner);
  with Result do
    begin
      Font := Screen.IconFont;
      fLoading := True;
      DoubleBuffered := True;
      pnlMain.DoubleBuffered := True;
      pnlBoolean.DoubleBuffered := True;
      Name := Format('%s_%d', [Result.ClassName, aOwner.ComponentCount]);
      Top := aOwner.ComponentCount * 20;
      Parent := aOwner;
      lblFolder.Caption := Format('Folder: %s', [TERM_TYPE_DISPLAY[aTerm.getTermType]]);
      lblFolder.Font.Style := [fsBold];
      lblBoolean.Font.Style := [fsBold];
      lblCommands.Font.Style := [fsBold];
      lblComponents.Font.Style := [fsBold];
      Align := alTop;
      try
        LoadTerm(aTerm);
      except
        on E: Exception do
          raise Exception.Create('Error loading term. ' + e.Message);
      end;
      fLoading := False;
    end;
end;
{ TfraEditTerm }
procedure TfraEditTerm.btnAssignVuidClick(Sender: TObject);
var
  aVUID: string;
begin
  aVUID := fTerm.getVuid;
  if InputQuery('Assign VUID to a Term', 'VUID:', aVUID) then
    if aVUID = '' then
      begin
        if MessageDlg(
          Format('Clear the VUID assignment for term ''%s''?', [fTerm.getTerm]),
          mtConfirmation, [mbYes, mbNo], 0) = mrYes then
          begin
            fTerm.setVUID('');
            fTerm.setChangeMade(True);
          end
      end
    else
      if MessageDlg(
        Format('Assign VUID ''%s'' to Term ''%s''', [aVUID, fTerm.getTerm]),
        mtConfirmation, [mbYes, mbNo], 0) = mrYes then
        begin
          fTerm.setVuid(aVUID);
          fTerm.setChangeMade(True);
        end;
end;
procedure TfraEditTerm.ChangeMade(Sender: TObject);
begin
  if not fLoading then
    with fTerm do
      begin
        setTerm(edtName.Text);
        setDisplayName(edtDisplayName.Text);
        setAbbreviation(edtAbbreviation.Text);
        setActive(cbxActive.Checked);
        setDescription(memDescription.Text);
        setHelpText(memHelpText.Text);
        if getDataType = dtPickList then
          setMultiSelectPicklist(cbxMultiSelect.Checked);
        if getDataType = dtBoolean then
          begin
            setBooleanValueTrue(edtBooleanValueTrue.Text);
            setBooleanValueFalse(edtBooleanValueFalse.Text);
          end
        else
          begin
            setBooleanValueTrue('');
            setBooleanValueFalse('');
          end;
        setChangeMade(True);
      end;
end;
procedure TfraEditTerm.LoadTerm(aTerm: ITerm);
var
  i: integer;
  tt: TTermType;
begin
  fTerm := aTerm;
  fTerm.Load;
  edtName.Text := fTerm.getTerm;
  edtName.Hint := Format(
    'Term Name: %s Internal GUID: %s',
    [fTerm.getTerm, fTerm.getId]);
  edtDisplayName.Text := fTerm.getDisplayName;
  edtAbbreviation.Text := fTerm.getAbbreviation;
  cbxActive.Checked := fTerm.getActive;
  cbxMultiSelect.Checked := fTerm.getMultiSelectPicklist;
  memDescription.Text := fTerm.getDescription;
  memHelpText.Text := fTerm.getHelpText;
  edtBooleanValueTrue.Text := fTerm.getBooleanValueTrue;
  edtBooleanValueFalse.Text := fTerm.getBooleanValueFalse;
  MyTerminology.LoadDataTypes(cbxDataType.Items);
  cbxDataType.ItemIndex := Ord(fTerm.getDataType);
  if fTerm.getTermType = ttObservation then
    begin
      cbxDataType.Visible := True;
      lblDataType.Visible := True;
      cbxMultiSelect.Visible := (fTerm.getDataType = dtPickList);
    end
  else
    cbxDataType.ItemIndex := -1;
  for i := pnlComponents.ComponentCount - 1 downto 0 do
    pnlComponents.Components[i].Free;
  if fTerm.getTermType = ttObservation then
    begin
      if fTerm.getDataType = dtNumeric then
        NewEditPairsFrame(pnlComponents, fTerm, ctUnit, ttUnit);
      if fTerm.getDataType = dtComplex then
        NewEditPairsFrame(pnlComponents, fTerm, ctChildTerm, ttObservation);
      if fTerm.getDataType = dtPicklist then
        NewEditPairsFrame(pnlComponents, fTerm, ctQualifier, ttDataValue);
      for tt := Low(TTermType) to High(TTermType) do
        if tt in [ttMethod, ttPosition, ttLocation, ttQuality, ttProduct] then
          NewEditPairsFrame(pnlComponents, fTerm, ctQualifier, tt);
    end;
  if fTerm.getTermType = ttUnit then
    newEditPairsFrame(pnlComponents, fTerm, ctConversion, ttUnit);
  pnlBoolean.Visible := fTerm.getDataType = dtBoolean;
  Height := pnlComponents.Top + pnlComponents.Height;
  lblComponents.Visible := (fTerm.getTermType = ttObservation) or (fTerm.getTermType = ttUnit);
  shpComponents.Visible := (fTerm.getTermType = ttObservation) or (fTerm.getTermType = ttUnit);
end;
procedure TfraEditTerm.btnToDoClick(Sender: TObject);
begin
  EditToDoListItem(fTerm);
end;
procedure TfraEditTerm.btnValidateClick(Sender: TObject);
var
  aQuickLog: TStringList;
begin
  try
    aQuickLog := TStringList.Create;
    fTerm.Validate;
    fTerm.LoadComponents(aQuickLog, ctError);
    if aQuickLog.Count = 0 then
      MessageDlg('No Errors found with term ''' + fTerm.getterm + '''.', mtInformation, [mbOk], 0)
    else
      ViewErrorLog(aQuickLog);
  finally
    FreeAndNil(aQuickLog);
  end;
end;
procedure TfraEditTerm.cbxDataTypeChange(Sender: TObject);
begin
  if MessageDlg(
    'WARNING!' + #13 + 'Changing the data type of this observation will result in ' +
    'the complete removal of all Clinical Data Model elements ' +
    '(i.e. Qualifier Pairs etc.) for this term.' + #13 + #13 +
    'Are you absolutely sure you want to change this term to data type ''' +
    TERM_DATATYPE[TDataType(cbxDataType.ItemIndex)] + '''?',
    mtWarning, [mbYes, mbNo, mbCancel], 0) = mrYes then
    try
      fTerm.ClearComponents;
      fTerm.setDataType(TDataType(cbxDataType.ItemIndex));
      if fTerm.getDataType = dtPicklist then
        fTerm.setMultiSelectPicklist(False);
      fTerm.Save;
    finally
      fTerm.Load; { Fresh copy from the XML }
      LoadTerm(fTerm); { Freshen up the UI }
      fTerm.setChangeMade(True); { DUH! }
      fOnUpdateTerm(fTerm); { Let the world know what we've done :) }
    end
  else
    cbxDataType.ItemIndex := Ord(TDataType(fTerm.getDataType));
end;
end.
