unit oTermChildPair;

interface

uses
  oTermComponent,
  uXML,
  XMLIntf,
  SysUtils,
  ComCtrls,
  Classes,
  iCPTerminology;

type
  TTermChildPair = class(TTermComponent, ITermChildPair)
  private
    fValueType: TChildValueType;
    fValueDelimeter: integer;
    fValueStartPosition: integer;
    fValueStopPosition: integer;
    fValueUnit: ITerm;
  public
    constructor Create(aParentTerm: ITerm; aChildTerm: ITerm);
    destructor Destroy; override;

    function getValueType: TChildValueType; stdcall;
    function getValueDelimiter: Char; stdcall;
    function getValueStartPosition: integer; stdcall;
    function getValueStopPosition: integer; stdcall;
    function getValueUnit: ITerm; stdcall;
    function getValueUnitID: string; stdcall;
    function getComponentDisplayName: string; override; stdcall;

    procedure setValueType(const aValue: TChildValueType); stdcall;
    procedure setValueDelimiter(const aValue: Char); stdcall;
    procedure setValueStartPosition(const aValue: integer); stdcall;
    procedure setValueStopPosition(const aValue: integer); stdcall;
    procedure setValueUnit(const aValue: ITerm); stdcall;

    procedure Load(aXMLNode: IXMLNode); override; stdcall;
    procedure Save(aXMLNode: IXMLNode); override; stdcall;
    procedure Validate; override; stdcall;
  end;

implementation

const
  {Default XML Tags for the properties}
  XML_VALUE_TYPE = 'VALUE_TYPE';
  XML_VALUE_DELIMITER = 'VALUE_DELIMITER';
  XML_VALUE_START_POSITION = 'VALUE_START_POSITION';
  XML_VALUE_STOP_POSITION = 'VALUE_STOP_POSITION';
  XML_VALUE_UNIT = 'VALUE_UNIT_ID';

  {Default values for the properties}
  DEFAULT_VALUE_TYPE = vtExtract;
  DEFAULT_VALUE_DELIMITER = 59; {Defaults to ';' Char(59)}
  DEFAULT_VALUE_START_POSITION = 1;
  DEFAULT_VALUE_STOP_POSITION = 1;

constructor TTermChildPair.Create(aParentTerm: ITerm; aChildTerm: ITerm);
begin
  inherited Create(ctChildTerm, aParentTerm, aChildTerm);
  fValueType := DEFAULT_VALUE_TYPE;
  fValueDelimeter := DEFAULT_VALUE_DELIMITER;
  fValueStartPosition := DEFAULT_VALUE_START_POSITION;
  fValueStopPosition := DEFAULT_VALUE_STOP_POSITION;
end;

destructor TTermChildPair.Destroy;
begin
  inherited;
end;

procedure TTermChildPair.Load(aXMLNode: IXMLNode);
begin
  try
    inherited Load(aXMLNode);
    fValueType := TChildValueType(XMLValueAsInt(aXMLNode, XML_VALUE_TYPE, Ord(DEFAULT_VALUE_TYPE)));
    fValueDelimeter := XMLValueAsInt(aXMLNode, XML_VALUE_DELIMITER, DEFAULT_VALUE_DELIMITER);
    fValueStartPosition := XMLValueAsInt(aXMLNode, XML_VALUE_START_POSITION, DEFAULT_VALUE_START_POSITION);
    fValueStopPosition := XMLValueAsInt(aXMLNode, XML_VALUE_STOP_POSITION, DEFAULT_VALUE_STOP_POSITION);
    fValueUnit := getParentTerm.getSibling(XMLValueAsStr(aXMLNode, XML_VALUE_UNIT, ''));
  except
    on E: Exception do
      raise ETerminologyLoadError.CreateFmt('Unable to load %s. Error: %s', [Self.ClassName, E.Message]);
  end;
end;

procedure TTermChildPair.Save(aXMLNode: IXMLNode);
begin
  try
    inherited Save(aXMLNode);
    aXMLNode.ChildValues[XML_VALUE_TYPE] := fValueType;
    aXMLNode.ChildValues[XML_VALUE_DELIMITER] := fValueDelimeter;
    aXMLNode.ChildValues[XML_VALUE_START_POSITION] := fValueStartPosition;
    aXMLNode.ChildValues[XML_VALUE_STOP_POSITION] := fValueStopPosition;
    if Assigned(fValueUnit) then
      aXMLNode.ChildValues[XML_VALUE_UNIT] := fValueUnit.getID
    else
      aXMLNode.ChildNodes.Delete(XML_VALUE_UNIT);

  except
    on E: Exception do
      raise ETerminologyLoadError.CreateFmt('Unable to save %s. Error: %s', [Self.ClassName, E.Message]);
  end;
end;

{ Private Getters }

function TTermChildPair.getComponentDisplayName: string;
begin
  Result := getComponentTerm.getTerm; //'Term Child Pair getComponentDisplayName';
end;

function TTermChildPair.getValueDelimiter: Char;
begin
  Result := Char(fValueDelimeter);
end;

function TTermChildPair.getValueStartPosition: integer;
begin
  Result := fValueStartPosition;
end;

function TTermChildPair.getValueStopPosition: integer;
begin
  Result := fValueStopPosition;
end;

function TTermChildPair.getValueType: TChildValueType;
begin
  Result := fValueType;
end;

function TTermChildPair.getValueUnit: ITerm;
begin
  Result := fValueUnit;
end;

function TTermChildPair.getValueUnitID: string;
begin
  if Assigned(fValueUnit) then
    Result := fValueUnit.getID
  else
    Result := '';
end;

{Private Setters}

procedure TTermChildPair.setValueDelimiter(const aValue: Char);
begin
  fValueDelimeter := Ord(aValue);
end;

procedure TTermChildPair.setValueStartPosition(const aValue: integer);
begin
  fValueStartPosition := aValue;
end;

procedure TTermChildPair.setValueStopPosition(const aValue: integer);
begin
  fValueStopPosition := aValue;
end;

procedure TTermChildPair.setValueType(const aValue: TChildValueType);
begin
  fValueType := aValue;
end;

procedure TTermChildPair.setValueUnit(const aValue: ITerm);
begin
  fValueUnit := aValue;
end;

procedure TTermChildPair.Validate;
begin
  inherited;
  if fValueStartPosition > fValueStopPosition then
    getParentTerm.NewError(etComponentSetting, getComponentTerm.getTerm + ' start position cannot be greater than stop position');
end;

end.

