unit oCPTIUDocument;

interface

uses
  Dialogs,
  Classes,
  ComCtrls,
  oCPTypes,
  oCliOTypes,
  oInterfaces,
  uCPXML,
  XMLDoc,
  SysUtils,
  XMLIntf;

type
  TCPTIUDocument = class(TInterfacedObject, ICPTIUDocument)
  private
    fDBConnection: IDBConnection;
    fUser: IUser;

    fID: string;
    fTitle: string;
    fTitleID: string;
    fLocation: string;
    fLocationID: string;
    fPatient: IPatient;
    fText: TStringList;
    fCosignerID: string;

    function getID: string; stdcall;
    function getLocation: string; stdcall;
    function getLocationID: string; stdcall;
    function getTitle: string; stdcall;
    function getTitleID: string; stdcall;
    function getPatient: IPatient; stdcall;
    function getUser: IUser; stdcall;

    procedure setLocation(const aValue: string); stdcall;
    procedure setLocationID(const aValue: string); stdcall;
    procedure setTitle(const aValue: string); stdcall;
    procedure setTitleID(const aValue: string); stdcall;
    procedure setPatient(const aValue: IPatient); stdcall;
    procedure setCosignerID(const aValue: string); stdcall;

    procedure ClearText; stdcall;
    procedure AppendText(aText: TStrings); overload; stdcall;
    procedure AppendText(aText: string); overload; stdcall;

    function Sign(aESIG: string): boolean; stdcall;
  public
    constructor Create(aDBConnection: IDBConnection; aUser: IUser);
    destructor Destroy; override;
  end;

implementation

uses
  Hash; {Broker utility to hash the E-Sig}

{ TCPTIUDocument }

constructor TCPTIUDocument.Create(aDBConnection: IDBConnection; aUser: IUser);
var
  aXMLDoc: IXMLDocument;
  aXMLRec: IXMLNode;
begin
  inherited Create;
  fDBConnection := aDBConnection;
  fUser := aUser;
  fText := TStringList.Create;

  aXMLDoc := NewXMLDocument;
  try
    fDBConnection.ExecQuery('GetTemporaryNoteID', [], aXMLDoc);
    aXMLRec := XMLDocumentFirstRecord(aXMLDoc);
    fID := XMLValueAsStr(aXMLRec, 'ID');
  except
    Free;
  end;
end;

destructor TCPTIUDocument.Destroy;
begin
  FreeAndNil(fText);
  inherited;
end;

procedure TCPTIUDocument.AppendText(aText: string);
begin
  fText.Add(aText);
end;

procedure TCPTIUDocument.AppendText(aText: TStrings);
begin
  fText.AddStrings(aText);
end;

procedure TCPTIUDocument.ClearText;
begin
  fText.Clear;
end;

function TCPTIUDocument.getID: string;
begin
  Result := fID;
end;

function TCPTIUDocument.getLocation: string;
begin
  Result := fLocation;
end;

function TCPTIUDocument.getLocationID: string;
begin
  Result := fLocationID;
end;

function TCPTIUDocument.getPatient: IPatient;
begin
  Result := fPatient;
end;

function TCPTIUDocument.getTitle: string;
begin
  Result := fTitle;
end;

function TCPTIUDocument.getTitleID: string;
begin
  Result := fTitleID;
end;

function TCPTIUDocument.getUser: IUser;
begin
  Result := fUser;
end;

procedure TCPTIUDocument.setCosignerID(const aValue: string);
begin
  fCosignerID := aValue;
end;

procedure TCPTIUDocument.setLocation(const aValue: string);
begin
  fLocation := aValue;
end;

procedure TCPTIUDocument.setLocationID(const aValue: string);
begin
  fLocationID := aValue;
end;

procedure TCPTIUDocument.setPatient(const aValue: IPatient);
begin
  fPatient := aValue;
end;

procedure TCPTIUDocument.setTitle(const aValue: string);
begin
  fTitle := aValue;
end;

procedure TCPTIUDocument.setTitleID(const aValue: string);
begin
  fTitleID := aValue;
end;

function TCPTIUDocument.Sign(aESIG: string): boolean;
var
  i: integer;
  aTextLines: array[0..10] of variant;
begin
  if fUser.IsValidElectronicSignature(aESIG) then
    try
      fDBConnection.ExecCommand('ClearNoteText', [fID]);
      while fText.Count > 0 do
        begin
          aTextLines[0] := fID;
          for i := 1 to High(aTextLines) do
            if fText.Count > 0 then
              begin
                aTextLines[i] := fText.Strings[0];
                fText.Delete(0);
              end
            else
              aTextLines[i] := '';
          {Append this bunch of text}
          fDBConnection.ExecCommand('AddTextToNote', aTextLines);
        end;
      Result := fDBConnection.ExecCommand('SignTIUDocument',
        [fID, fPatient.getDFN, getTitleID, getLocationID, Encrypt(aESIG), fCosignerID]);
    except
      raise Exception.Create('Error Filing Note!');
    end
  else
    Result := False;
end;

end.

