unit mLookup;
================================================================================
*       Application:  Clinical Observations
*       Revision:     $Revision: 7 $  $Modtime: 7/24/09 9:55a $
*       Developer:      
*       Site:         Hines OIFO
*       Description:  Drop on frame for managing a lookup field on a form.
*       Notes:
================================================================================
*       $Archive: /CP Modernization/CliO/source/TLookupFrame/mLookup.pas $
================================================================================
interface
uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ExtCtrls,
  Buttons,
  VA508AccessibilityManager;
type
  {Generic interface for whatever objects need to be placed in the fResults list}
  IFetchItem = interface
    ['{B66BF7A1-3E58-4281-8774-49DEA679AECC}']
    function getInternalValue: string; stdcall;
    function getExternalValue: string; stdcall;
  end;
  {Generic object to implement a list for this widget}
  TFetchItem = class(TInterfacedObject, IFetchItem)
  public
    InternalValue: string;
    ExternalValue: string;
    constructor Create;
    destructor Destroy; override;
    function _AddRef: Integer; stdcall;
    function _Release: Integer; stdcall;
    function getInternalValue: string; stdcall;
    function getExternalValue: string; stdcall;
  end;
  {Must be implemented in the using application}
  TFetchData = procedure(aPartial: string; aResult: TStrings) of object;
  TFetchHelp = procedure of object;
  TfraLookup = class(TFrame)
    tmrLookup: TTimer;
    ComboBox1: TComboBox;
    pnl: TPanel;
    procedure tmrLookupTimer(Sender: TObject);
    procedure edtLookupKeyPress(Sender: TObject; var Key: Char);
    procedure edtLookupKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure edtLookupExit(Sender: TObject);
    procedure ComboBox1DropDown(Sender: TObject);
    procedure FrameEnter(Sender: TObject);
    procedure ComboBox1Enter(Sender: TObject);
  protected
    f508Value: string;
    fResults: TStringList;
    fInternalValue: string;
    fExternalValue: string;
    fOnFetchData: TFetchData;
    fOnFetchHelp: TFetchHelp;
    fLoading: boolean;
    procedure ClearItems;
  public
    procedure Initialize(aInternalValue, aExternalValue: string; aFetchData: TFetchData;
      aFetchHelp: TFetchHelp; ALabel: TLabel);
    property InternalValue: string read fInternalValue;
    property ExternalValue: string read fExternalValue;
  end;
function NewFraLookup(aOwner: TWinControl): TfraLookup;
procedure AddItem(aInternalValue, aExternalValue: string; aList: TStrings);
implementation
uses
  fLookup,
  IniFiles,
  VA508AccessibilityRouter,
  uCP_GUI;
const
  MyKeys = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvsxyz0123456789`-=[]\;'',./~!@#$%^&*()_+{}|:"<>?';
  NoData = 'No data found matching ''%s''.' + #13 + #13 + 'Make sure your spelling and character case are correct';
var
  ObjCount: integer = 0;
  ObjID: integer = 0;
{$R *.dfm}
function NewFraLookup(aOwner: TWinControl): TfraLookup;
begin
  Result := TfraLookup.Create(aOwner);
  Result.Parent := aOwner;
  Result.Visible := true;
end;
procedure AddItem(aInternalValue, aExternalValue: string; aList: TStrings);
var
  aItem: TFetchItem;
begin
  aItem := TFetchItem.Create;
  aItem.InternalValue := aInternalValue;
  aItem.ExternalValue := aExternalValue;
  aList.AddObject(aExternalValue, aItem);
end;
{ TfraLookup }
procedure TfraLookup.Initialize(aInternalValue, aExternalValue: string;
  aFetchData: TFetchData; aFetchHelp: TFetchHelp;  ALabel: TLabel);
//var
//  Item: TVA508AccessibilityItem;
begin
  fInternalValue := aInternalValue;
  fExternalValue := aExternalValue;
  fOnFetchData := aFetchData;
  fOnFetchHelp := aFetchHelp;
  fLoading := False;
  ComboBox1.Text := aExternalValue;
  tmrLookup.Interval := 350;
  tmrLookup.Enabled := False;
  f508Value := ALabel.Caption;
//  Item := A508Manager.AccessData.Add;
//  Item.Component := ComboBox1;
//  Item.AccessLabel := ALabel;
//
//  A508Manager.RefreshComponents;
end;
procedure TfraLookup.tmrLookupTimer(Sender: TObject);
//var
//  pt: TPoint;
//  i: integer;
//  aItem: IFetchItem;
//  obj: TFetchItem;
begin
  tmrLookup.Enabled := False;
  fLoading := True;
  try
    ClearItems;
    fOnFetchData(ComboBox1.Text, ComboBox1.Items);
    {
    for i := 0 to ComboBox1.Items.Count - 1 do
      if AnsiCompareText(
        copy(ComboBox1.Items[i], 1, Length(ComboBox1.Text)),
        ComboBox1.Text) = 0 then
        begin
          ComboBox1.ItemIndex := i;
          ComboBox1.DroppedDown := True;
          Exit;
        end;
    }
    case ComboBox1.Items.Count of
      0:
        begin
          MessageDlg('No items found to match ''' + ComboBox1.Text + '''.', mtError, [mbOk], 0);
          ComboBox1.ItemIndex := -1;
          ComboBox1.Text := fExternalValue;
        end;
      1:
        begin
          ComboBox1.ItemIndex := 0;
          ComboBox1.Text := ComboBox1.Items[0];
        end
    else
      begin
        ComboBox1.DroppedDown := True;
        ComboBox1.ItemIndex := 0;
      end;
    end;
  except
    MessageDlg('Error encounted lookup up entries.', mtError, [mbOk], 0);
  end;
  fLoading := False;
end;
procedure TfraLookup.edtLookupKeyPress(Sender: TObject; var Key: Char);
begin
  tmrLookup.Enabled := False;
  if Pos(Key, MyKeys) > 0 then
    tmrLookup.Enabled := True
  else if Ord(Key) = 8 then
    if (Length(ComboBox1.Text) - (ComboBox1.SelLength)) > 1 then
      tmrLookup.Enabled := True
    else
      begin {You're deleting everything, turn off the timer and set the internals}
        tmrLookup.Enabled := False;
        fInternalValue := '';
        fExternalValue := '';
      end;
end;
procedure TfraLookup.edtLookupKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
var
  NewKey: Char;
begin
  if Key = 46 then {OK, we've pressed the delete key, let act like it was a backspace in KeyPress}
    begin
      NewKey := Chr(8);
      edtLookupKeyPress(Sender, NewKey);
    end;
  if Key = VK_F1 then
    if Assigned(fOnFetchHelp) then
      fOnFetchHelp
    else
      MessageDlg('Sorry, no help available.', mtInformation, [mbOk], 0);
end;
procedure TfraLookup.FrameEnter(Sender: TObject);
begin
  ComboBox1.SetFocus;
end;
{ TFetchItem }
constructor TFetchItem.Create;
begin
  inherited Create;
end;
destructor TFetchItem.Destroy;
begin
  inherited;
end;
function TFetchItem.getExternalValue: string;
begin
  Result := ExternalValue;
end;
function TFetchItem.getInternalValue: string;
begin
  Result := InternalValue;
end;
function TFetchItem._AddRef: Integer;
begin
  Result := -1;
end;
function TFetchItem._Release: Integer;
begin
  Result := -1;
end;
procedure TfraLookup.edtLookupExit(Sender: TObject);
var
  aFetchItem: IFetchItem;
begin
  if ComboBox1.Text = '' then
    begin
      fExternalValue := '';
      fInternalValue := '';
      ClearItems;
      tmrLookup.Enabled := False;
    end
  else if tmrLookup.Enabled then {Their has been activity, fire for effect}
    tmrLookupTimer(Sender)
  else if ComboBox1.ItemIndex > -1 then
    if ComboBox1.Items.Objects[ComboBox1.ItemIndex].GetInterface(IFetchItem, aFetchItem) then
      begin
        fExternalValue := aFetchItem.getExternalValue;
        fInternalValue := aFetchItem.getInternalValue;
      end;
  aFetchItem := nil; {Ensures this interface is released appropriately}
end;
procedure TfraLookup.ClearItems;
begin
  while ComboBox1.Items.Count > 0 do
    begin
      if Assigned(ComboBox1.Items.Objects[0]) then
        ComboBox1.Items.Objects[0].Free;
      ComboBox1.Items.Delete(0);
    end;
end;
procedure TfraLookup.ComboBox1DropDown(Sender: TObject);
begin
  if not fLoading and (ComboBox1.Text <> '') and (tmrLookup.Enabled) then
    tmrLookupTimer(Sender);
  SetDropDownWidth(ComboBox1);
end;
procedure TfraLookup.ComboBox1Enter(Sender: TObject);
begin
  GetScreenReader.Speak(f508Value);
end;
end.
